package ch.phoenix.oss.quarkus.commons.audit;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertAll;
import static org.mockito.Mockito.when;

import ch.phoenix.oss.quarkus.commons.tracing.TracingService;
import io.quarkus.narayana.jta.QuarkusTransaction;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.mockito.InjectSpy;
import jakarta.inject.Inject;
import jakarta.persistence.EntityManager;
import java.time.Instant;
import org.hibernate.envers.AuditReaderFactory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

@QuarkusTest
class AuditedPanacheEntityBaseTest {

    @Inject
    EntityManager entityManager;

    @InjectSpy
    TracingService tracingService;

    @SuppressWarnings("Convert2MethodRef")
    @BeforeEach
    void setup() {
        QuarkusTransaction.requiringNew().run(() -> TestEntity2.deleteAll());
    }

    @Test
    void persistAndUpdate() {
        when(tracingService.getRequestId()).thenReturn("00000000-0000-0000-0000-000000000000");
        when(tracingService.getSpanId()).thenReturn("0000000000000000");
        when(tracingService.getTraceId()).thenReturn("00000000000000000000000000000000");
        when(tracingService.getActor()).thenReturn("unknown");
        when(tracingService.getClientIp()).thenReturn("unknown");

        var now = Instant.now();

        QuarkusTransaction.requiringNew().run(() -> {
            var entity = new TestEntity2("something");
            entity.persist();
        });

        QuarkusTransaction.requiringNew().run(() -> {
            var entity = TestEntity2.findBySomething("something");
            assertAll(
                    () -> assertThat(entity.createdAt)
                            .as("createdAt should be after or equal to expected value")
                            .isAfterOrEqualTo(now),
                    () -> assertThat(entity.lastUpdatedAt)
                            .as("lastUpdatedAt should be equal to createdAt")
                            .isEqualTo(entity.createdAt));

            var auditReader = AuditReaderFactory.get(entityManager);

            var revisions = auditReader.getRevisions(TestEntity2.class, entity.id);
            assertThat(revisions).hasSize(1);

            var revInfo = entityManager.find(Revision.class, revisions.getFirst());
            assertThat(revInfo).isNotNull();

            assertAll(
                    () -> assertThat(revInfo.timestamp)
                            .as("revision timestamp should be equal to entity's createdAt timestamp")
                            .isEqualTo(entity.createdAt),
                    () -> assertThat(revInfo.actor)
                            .as("actor should match expected value")
                            .isEqualTo("unknown"),
                    () -> assertThat(revInfo.traceId)
                            .as("traceId should match expected value")
                            .isEqualTo("00000000000000000000000000000000"),
                    () -> assertThat(revInfo.spanId)
                            .as("spanId should match expected value")
                            .isEqualTo("0000000000000000"),
                    () -> assertThat(revInfo.requestId)
                            .as("requestId should match expected value")
                            .isEqualTo("00000000-0000-0000-0000-000000000000"),
                    () -> assertThat(revInfo.clientIp)
                            .as("clientIp should match expected value")
                            .isEqualTo("unknown"),
                    () -> assertThat(revInfo.hostName)
                            .as("hostName should not be blank")
                            .isNotBlank());
        });

        QuarkusTransaction.requiringNew().run(() -> {
            var entity = TestEntity2.findBySomething("something");
            entity.something = "else";
        });

        QuarkusTransaction.requiringNew().run(() -> {
            var entity = TestEntity2.findBySomething("else");
            assertAll(() -> assertThat(entity.createdAt)
                    .as("createdAt should be before lastUpdatedAt")
                    .isBefore(entity.lastUpdatedAt));

            var auditReader = AuditReaderFactory.get(entityManager);

            var revisions = auditReader.getRevisions(TestEntity2.class, entity.id);
            assertThat(revisions).hasSize(2);

            Revision revInfo = Revision.findById(revisions.getLast());
            assertThat(revInfo).isNotNull();

            assertAll(
                    () -> assertThat(revInfo.timestamp)
                            .as("revision timestamp should not be equal to entity's createdAt")
                            .isNotEqualTo(entity.createdAt),
                    () -> assertThat(revInfo.timestamp)
                            .as("revision timestamp should be equal to entity's lastUpdatedAt")
                            .isEqualTo(entity.lastUpdatedAt),
                    () -> assertThat(revInfo.actor)
                            .as("actor should match expected value")
                            .isEqualTo("unknown"),
                    () -> assertThat(revInfo.traceId)
                            .as("traceId should match expected value")
                            .isEqualTo("00000000000000000000000000000000"),
                    () -> assertThat(revInfo.spanId)
                            .as("spanId should match expected value")
                            .isEqualTo("0000000000000000"),
                    () -> assertThat(revInfo.requestId)
                            .as("requestId should match expected value")
                            .isEqualTo("00000000-0000-0000-0000-000000000000"),
                    () -> assertThat(revInfo.clientIp)
                            .as("clientIp should match expected value")
                            .isEqualTo("unknown"),
                    () -> assertThat(revInfo.hostName)
                            .as("hostName should not be blank")
                            .isNotBlank());
        });
    }
}
