package ch.phoenix.oss.quarkus.commons.random;

import java.math.BigInteger;
import java.util.Random;

/**
 * This class provides methods to generate random numbers.
 * The methods are similar to those in the Random class, but
 * this class uses a RandomProvider to obtain the Random
 * instance used for generating the random numbers.
 *
 * @see Random
 * @see RandomProvider
 */
public interface RandomNumberGenerator {

    /**
     * Generates a random integer. Uses a Random instance
     * obtained from the RandomProvider.
     *
     * @return a random integer
     */
    int generateInt();

    /**
     * Generates a random integer between 0 (inclusive) and the
     * specified bound (exclusive). Uses a Random instance
     * obtained from the RandomProvider.
     *
     * @param bound the upper bound (exclusive). Must be positive.
     * @return a random integer
     */
    int generateInt(int bound);

    /**
     * Generates a random integer between the specified origin
     * (inclusive) and the specified bound (exclusive). Uses a
     * Random instance obtained from the RandomProvider.
     *
     * @param origin the lower bound (inclusive)
     * @param bound  the upper bound (exclusive). Must be greater than origin.
     * @return a random integer
     */
    int generateInt(int origin, int bound);

    /**
     * Generates a random long. Uses a Random instance obtained
     * from the RandomProvider.
     *
     * @return a random long
     */
    long generateLong();

    /**
     * Generates a random long between 0 (inclusive) and the
     * specified bound (exclusive). Uses a Random instance
     * obtained from the RandomProvider.
     *
     * @param bound the upper bound (exclusive). Must be positive.
     * @return a random long
     */
    long generateLong(long bound);

    /**
     * Generates a random long between the specified origin
     * (inclusive) and the specified bound (exclusive). Uses a
     * Random instance obtained from the RandomProvider.
     *
     * @param origin the lower bound (inclusive)
     * @param bound  the upper bound (exclusive). Must be greater than origin.
     * @return a random long
     */
    long generateLong(long origin, long bound);

    /**
     * Generates a random float between 0.0 (inclusive) and 1.0 (exclusive).
     * Uses a Random instance obtained from the RandomProvider.
     *
     * @return a random float
     */
    float generateFloat();

    /**
     * Generates a random float between 0.0 (inclusive) and the
     * specified bound (exclusive). Uses a Random instance obtained
     * from the RandomProvider.
     *
     * @param bound the upper bound (exclusive). Must be positive.
     * @return a random float
     */
    float generateFloat(float bound);

    /**
     * Generates a random float between the specified origin (inclusive)
     * and the specified bound (exclusive). Uses a Random instance
     * obtained from the RandomProvider.
     *
     * @param origin the lower bound (inclusive)
     * @param bound  the upper bound (exclusive). Must be greater than origin.
     * @return a random float
     */
    float generateFloat(float origin, float bound);

    /**
     * Generates a random double between 0.0 (inclusive) and 1.0 (exclusive).
     * Uses a Random instance obtained from the RandomProvider.
     *
     * @return a random double
     */
    double generateDouble();

    /**
     * Generates a random double between 0.0 (inclusive) and the
     * specified bound (exclusive). Uses a Random instance obtained
     * from the RandomProvider.
     *
     * @param bound the upper bound (exclusive). Must be positive.
     * @return a random double
     */
    double generateDouble(double bound);

    /**
     * Generates a random double between the specified origin (inclusive)
     * and the specified bound (exclusive). Uses a Random instance
     * obtained from the RandomProvider.
     *
     * @param origin the lower bound (inclusive)
     * @param bound  the upper bound (exclusive). Must be greater than origin.
     * @return a random double
     */
    double generateDouble(double origin, double bound);

    /**
     * Generates a random BigInteger with the specified number of bits.
     * Uses a Random instance obtained from the RandomProvider.
     *
     * @param numBits the number of bits of the BigInteger to generate
     * @return a random BigInteger
     */
    BigInteger generateBigInteger(int numBits);

    /**
     * Generates a random BigInteger with the specified number of bits
     * using the specified Random instance.
     *
     * @param numBits the number of bits of the BigInteger to generate
     * @param random  the Random instance to use
     * @return a random BigInteger
     */
    BigInteger generateBigInteger(int numBits, Random random);
}
