package ch.phoenix.oss.quarkus.commons.random;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

import io.quarkus.test.junit.QuarkusTest;
import jakarta.inject.Inject;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.SecureRandom;
import java.security.Security;
import org.junit.jupiter.api.Test;

@QuarkusTest
@SuppressWarnings("removal")
class RandomProviderImplTest {

    public static final String SECURE_RANDOM_STRONG_ALGORITHMS = "securerandom.strongAlgorithms";

    @Inject
    RandomProviderImpl provider;

    @Test
    void get() {
        assertThat(provider.get())
                .as("Random instance should not be null")
                .isNotNull()
                .isInstanceOf(SecureRandom.class);
    }

    @Test
    void getWhenNoSuchAlgorithm() {
        var algo = getSecurityProperty();

        setSecurityProperty("non-existent-algorithm");

        try {
            assertThatThrownBy(() -> provider.get())
                    .as("Should throw IllegalStateException when algorithm is not found")
                    .isInstanceOf(IllegalStateException.class)
                    .hasMessageContaining("Unable to obtain strong SecureRandom instance");
        } finally {
            setSecurityProperty(algo);
        }
    }

    private static String getSecurityProperty() {
        return AccessController.doPrivileged(
                (PrivilegedAction<String>) () -> Security.getProperty(SECURE_RANDOM_STRONG_ALGORITHMS));
    }

    private static void setSecurityProperty(String datum) {
        AccessController.doPrivileged((PrivilegedAction<Void>) () -> {
            Security.setProperty(SECURE_RANDOM_STRONG_ALGORITHMS, datum);
            return (Void) null;
        });
    }
}
