package ch.phoenix.oss.quarkus.commons.uuid;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

import io.quarkus.test.junit.QuarkusTest;
import jakarta.inject.Inject;
import java.util.Random;
import java.util.UUID;
import java.util.stream.Stream;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

@QuarkusTest
class UUIDGeneratorImplTest {

    private static final UUID NAMESPACE_DNS = UUID.fromString("6ba7b810-9dad-11d1-80b4-00c04fd430c8");
    private static final UUID NAMESPACE_URL = UUID.fromString("6ba7b811-9dad-11d1-80b4-00c04fd430c8");
    private static final UUID NAMESPACE_OID = UUID.fromString("6ba7b812-9dad-11d1-80b4-00c04fd430c8");
    private static final UUID NAMESPACE_X500 = UUID.fromString("6ba7b814-9dad-11d1-80b4-00c04fd430c8");

    private static final String V5_NAME = "test";
    private static final UUID EXPECTED_V5_DNS = UUID.fromString("4be0643f-1d98-573b-97cd-ca98a65347dd");
    private static final UUID EXPECTED_V5_URL = UUID.fromString("da5b8893-d6ca-5c1c-9a9c-91f40a2a3649");
    private static final UUID EXPECTED_V5_OID = UUID.fromString("b428b5d9-df19-5bb9-a1dc-115e071b836c");
    private static final UUID EXPECTED_V5_X500 = UUID.fromString("63a3ab2b-61b8-5b04-ae2f-70d3875c6e97");

    @Inject
    UUIDGeneratorImpl generator;

    @Test
    void generateV4() {
        assertThat(generator.generateV4())
                .as("Generated UUID should not be null")
                .isNotNull();
    }

    @Test
    void generateV4WithLocalRandom() {
        var expected = UUID.fromString("b22f5354-f5ed-4f02-bca8-8882a6ec4750");

        var actual = generator.generateV4(getRandom());

        assertThat(actual).as("Generated UUID should match expected value").isEqualTo(expected);
    }

    @Test
    void generateV5() {
        var expected = UUID.fromString("a94a8fe5-ccb1-5ba6-9c4c-0873d391e987");

        var actual = generator.generateV5("test");

        assertThat(actual).as("Generated UUID should match expected value").isEqualTo(expected);
    }

    public static Stream<Arguments> generateV5WithNamespace() {
        return Stream.of(
                Arguments.of(NAMESPACE_DNS, V5_NAME, EXPECTED_V5_DNS),
                Arguments.of(NAMESPACE_URL, V5_NAME, EXPECTED_V5_URL),
                Arguments.of(NAMESPACE_OID, V5_NAME, EXPECTED_V5_OID),
                Arguments.of(NAMESPACE_X500, V5_NAME, EXPECTED_V5_X500));
    }

    @MethodSource
    @ParameterizedTest
    void generateV5WithNamespace(UUID namespace, String name, UUID expected) {
        var actual = generator.generateV5(namespace, name);

        assertThat(actual).as("Generated UUID should match expected value").isEqualTo(expected);
    }

    @Test
    void generateV5WithDNSNamespace() {
        var actual = generator.generateV5WithDNSNamespace(V5_NAME);

        assertThat(actual).as("Generated UUID should match expected value").isEqualTo(EXPECTED_V5_DNS);
    }

    @Test
    void generateV5WithURLNamespace() {
        var actual = generator.generateV5WithURLNamespace(V5_NAME);

        assertThat(actual).as("Generated UUID should match expected value").isEqualTo(EXPECTED_V5_URL);
    }

    @Test
    void generateV5WithOIDNamespace() {
        assertThat(generator.generateV5WithOIDNamespace(V5_NAME))
                .as("Generated UUID should match expected value")
                .isEqualTo(EXPECTED_V5_OID);
    }

    @Test
    void generateV5WithX500Namespace() {
        assertThat(generator.generateV5WithX500Namespace(V5_NAME))
                .as("Generated UUID should match expected value")
                .isEqualTo(EXPECTED_V5_X500);
    }

    private static Random getRandom() {
        return new Random(24353L);
    }
}
